@@ -78,6 +78,7 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
             pub use checkout_core::internal_context::InternalContext;
 
             use std::env;
+            use checkout_core::cloud_pubsub;
             use checkout_core::dropshot;
             use checkout_core::dropshot::endpoint;
             use checkout_core::dropshot::RequestContext;
@@ -121,6 +122,52 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                 pub context_provider: Arc<super::#server_type_name>,
             }
 
+            #[derive(Debug)]
+            struct Message(String);
+
+            impl cloud_pubsub::FromPubSubMessage for Message {
+                fn from(message: cloud_pubsub::EncodedMessage) -> Result<Self, cloud_pubsub::error::Error> {
+                    match message.decode() {
+                        Ok(bytes) => Ok(Message(String::from_utf8_lossy(&bytes).into_owned())),
+                        Err(e) => Err(cloud_pubsub::error::Error::from(e)),
+                    }
+                }
+            }
+
+            pub async fn start_payment_processing(
+                client: Arc<cloud_pubsub::Client>,
+                subscription_name: String,
+                internal_ctx: InternalContext,
+                context_provider: Arc<super::#server_type_name>,
+            ) {
+                let subscription = client.subscribe(subscription_name);
+
+                while subscription.client().is_running() {
+                    match subscription.get_messages::<Message>().await {
+                        Ok(messages) => {
+                            for (result, ack_id) in messages {
+                                match result {
+                                    Ok(message) => {
+                                        println!("Received: {:?}", message);
+                                        match context_provider.new_context().await {
+                                            Ok(ctx) => {
+                                                let processing_result = process_handler(&internal_ctx, &mut ctx, message.0).await;
+                                                if processing_result.is_ok() {
+                                                    subscription.acknowledge_messages(vec![ack_id]).await;
+                                                }
+                                            }
+                                            Err(_) => {}
+                                        }
+                                    }
+                                    Err(e) => eprintln!("Failed to parse message: {}", e),
+                                }
+                            }
+                        }
+                        Err(e) => eprintln!("Failed to pull PubSub messages: {}", e),
+                    }
+                }
+            }
+
             #[endpoint {
                 method = POST,
                 path = "/checkout",
